#ifndef bqt_mixerstream_hh
#define bqt_mixerstream_hh

#include <vector>
#include <netinet/in.h> /* for sockaddr_in */

#include "soundtypes.h"

class Stream
{
protected:
    int pid, fd;
    bool problem;
public:
    Stream(int serverfd, int pipefd);
    ~Stream();
    inline int getfd() const { return fd; }
    inline bool problems() const { return problem; }
};

class SockData
{
protected:
    struct sockaddr_in They;
    unsigned AddrLen;
public:
    SockData(class Waiter &);
    SockData(const struct sockaddr_in &t, unsigned al) : They(t), AddrLen(al) { }
};

class Waiter : public SockData
{
    int fd;
    char type;
public:
    Waiter(int p, const struct sockaddr_in &t, unsigned al) : SockData(t, al), fd(p), type(0) { }
    inline int getfd() const { return fd; }
    void Evaluate()
    {
    	int c = read(fd, &type, 1);
    	if(c <= 0)return;
    }
    inline bool isinput()  const { return type=='S'; /* stream */ }
    inline bool isoutput() const { return type=='M'; /* monitor */ }
};

class MixerInput : public Stream, public SockData
{
    uint8 Buf[32768];
    unsigned bufsiz;
public:
	MixerInput(const Waiter &w) : Stream(w.getfd(), 0), SockData(w), bufsiz(0) { }
    
    /* Gets new data if possible */
    void Evaluate();

    /* Number of elements available */
    inline unsigned size() const { return bufsiz >> 2; }
    inline const stent *ptr() const { return (const stent *)&Buf[0]; }
    
    /* Acknowledge stereo elements (throw them out) */
    void Eat(unsigned count);
    
};

class MixerOutput : public Stream, public SockData
{
    vector<uint8> Buf;
    unsigned head;
public:
	MixerOutput(const Waiter &w) : Stream(w.getfd(), 1), SockData(w), head(0) { }
    /* Add these to buffer */
    void Send(const stent *s, unsigned n);
    /* Send what we have, at least try */
    void Evaluate();
    /* Something to send? */
    unsigned pending() const { return Buf.size()-head; }
    inline bool panic() const { return Buf.size() > 65536; }
};

#endif
